use std::sync::Arc;

use axum::{
    async_trait,
    extract::{FromRef, FromRequestParts},
    http::request::Parts,
};
use jsonwebtoken::{DecodingKey, EncodingKey};
use sqlx::{postgres::Postgres, Pool};

#[derive(Clone, FromRef)]
pub struct AppState {
    pub pool: Pool<Postgres>,
    pub hbs: handlebars::Handlebars<'static>,
    pub encoding_key: EncodingKey,
    pub decoding_key: DecodingKey,
}

#[async_trait]
impl<S> FromRequestParts<S> for AppState
where
    Self: FromRef<S>,
    S: Send + Sync,
{
    type Rejection = &'static str;

    async fn from_request_parts(_parts: &mut Parts, state: &S) -> Result<Self, Self::Rejection> {
        Ok(Self::from_ref(state))
    }
}

impl FromRef<Arc<AppState>> for AppState {
    fn from_ref(input: &Arc<AppState>) -> Self {
        (**input).clone()
    }
}
